@@ -5,7 +5,17 @@ import posixpath
 import shutil
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    TypeVar,
+)
 
 import yaml
 from dvc_data.hashfile.db.local import LocalHashFileDB
@@ -16,6 +26,7 @@ from tqdm import tqdm
 from dql.client import Client
 from dql.data_storage import AbstractDataStorage
 from dql.dataset import DatasetRecord, DatasetRow
+from dql.dataset import Status as DatasetStatus
 from dql.listing import Listing
 from dql.node import NodeWithPath
 from dql.storage import Status, Storage
@@ -28,6 +39,8 @@ DATASET_FILE_SUFFIX = ".dql"
 
 TTL_INT = 4 * 60 * 60
 
+T = TypeVar("T")
+
 
 class PendingIndexingError(Exception):
     pass
@@ -503,13 +516,14 @@ class Catalog:
 
         return node_groups
 
-    def upsert_shadow_dataset(
+    def create_shadow_dataset(
         self,
         name: str,
         sources: List[str],
         client_config=None,
         recursive=False,
-    ) -> None:
+        populate=True,
+    ) -> DatasetRecord:
         """
         Creates shadow dataset in DB if it doesn't exist and updates it with
         entries from sources.
@@ -519,12 +533,24 @@ class Catalog:
             s3://bucket_name/image_*
         """
         client_config = client_config or {}
-        self.data_storage.create_shadow_dataset(name)
-        for source in sources:
-            client, path = self.parse_url(source, **client_config)
-            self.data_storage.insert_into_shadow_dataset(
-                name, client.uri, path, recursive=recursive
-            )
+        dataset = self.data_storage.create_shadow_dataset(name, create_rows=populate)
+        assert dataset
+        if not populate:
+            # returning empty dataset without dataset rows table and data inside
+            return dataset
+
+        final_status = DatasetStatus.FAILED
+        try:
+            for source in sources:
+                client, path = self.parse_url(source, **client_config)
+                self.data_storage.insert_into_shadow_dataset(
+                    name, client.uri, path, recursive=recursive
+                )
+            final_status = DatasetStatus.COMPLETE
+        finally:
+            self.data_storage.update_dataset_status(dataset, final_status)
+
+        return dataset
 
     def register_shadow_dataset(
         self,
@@ -717,6 +743,13 @@ class Catalog:
 
         return dst
 
+    def read_object(
+        self, row: DatasetRow, reader: Callable[[Any], T], **config: Any
+    ) -> T:
+        client, _ = self.parse_url(row.source, **config)
+        with client.open(row.path_str) as f:
+            return reader(f)
+
     def ls(
         self,
         sources: List[str],
@@ -833,7 +866,7 @@ class Catalog:
         )
 
         if create_dataset:
-            self.upsert_shadow_dataset(
+            self.create_shadow_dataset(
                 output, sources, client_config, recursive=recursive
             )
 
