@@ -189,6 +189,19 @@ def get_parser() -> ArgumentParser:
         "register", parents=[parent_parser], help="Register a dataset"
     )
     parse_register.add_argument("name", type=str, help="Dataset name")
+    parse_register.add_argument(
+        "--registered-name",
+        action="store",
+        default="",
+        help="Registered dataset name",
+    )
+    parse_register.add_argument(
+        "--version",
+        action="store",
+        default=None,
+        type=int,
+        help="New registered dataset version",
+    )
     parse_register.add_argument(
         "--description",
         action="store",
@@ -416,7 +429,13 @@ def get_parser() -> ArgumentParser:
     add_storage_parser = subp.add_parser(
         "add-storage", parents=[parent_parser], help="Register local path as storage"
     )
-    add_storage_parser.add_argument("uri", type=str, help="Source URI")
+    add_storage_parser.add_argument("uri", type=str, help="Source URI or path")
+    add_storage_parser.add_argument(
+        "--symlinks",
+        default=False,
+        action="store_true",
+        help="Use symlinks instead of full copies when downloading from storage",
+    )
 
     subp.add_parser(
         "find-stale-storages",
@@ -466,8 +485,8 @@ def cp(sources, output, **kwargs):
 def register(name, **kwargs):
     data_storage = SQLiteDataStorage()
     catalog = Catalog(data_storage)
-    catalog.register(name, **kwargs)
-    print(f"Registered dataset {name}, version 1 created")
+    catalog.register_shadow_dataset(name, **kwargs)
+    print(f"Registered dataset {name}")
 
 
 def edit_dataset(name, **kwargs):
@@ -685,14 +704,14 @@ def index(sources, catalog: Optional[Catalog] = None, **kwargs):
     catalog.index(sources, **kwargs)
 
 
+def add_storage(uri: str, symlinks: bool = False) -> None:
+    Catalog(SQLiteDataStorage()).add_storage(uri, symlinks=symlinks)
+
+
 def find_stale_storages():
     Catalog(SQLiteDataStorage()).find_stale_storages()
 
 
-def add_storage(uri: str):
-    Catalog(SQLiteDataStorage()).add_storage(uri)
-
-
 def completion(shell: str) -> str:
     return shtab.complete(
         get_parser(),
@@ -762,6 +781,8 @@ def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
                 args.name,
                 description=args.description,
                 labels=args.labels,
+                registered_name=args.registered_name,
+                version=args.version,
             )
         elif args.command == "edit-dataset":
             edit_dataset(
@@ -830,7 +851,7 @@ def main(argv: Optional[List[str]] = None) -> int:  # noqa: C901
                 client_config=client_config,
             )
         elif args.command == "add-storage":
-            add_storage(args.uri)
+            add_storage(args.uri, symlinks=args.symlinks)
         elif args.command == "completion":
             print(completion(args.shell))
         elif args.command == "find-stale-storages":
