@@ -5,7 +5,7 @@ import posixpath
 from abc import abstractmethod
 from datetime import datetime
 from shutil import copy2
-from typing import TYPE_CHECKING, ClassVar, Iterator, Tuple, Type
+from typing import TYPE_CHECKING, Any, ClassVar, Iterator, Tuple, Type
 
 from botocore.exceptions import ClientError
 from dvc_data.hashfile.db.local import LocalHashFileDB
@@ -234,3 +234,6 @@ class FSSpecClient(Client):
         except (NotImplementedError, ReflinkImpossibleError):
             # Default to copy if reflinks are not supported
             copy2(src, dst)
+
+    def open(self, path: str, mode="rb") -> Any:
+        return self.fs.open(self.get_full_path(path), mode=mode)
