@@ -2,19 +2,36 @@ import hashlib
 import json
 import logging
 import operator
+import os
 import posixpath
 import sqlite3
 from datetime import MAXYEAR, MINYEAR, datetime, timezone
 from functools import partial, wraps
 from itertools import groupby
 from time import sleep
-from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 import sqlalchemy
 from sqlalchemy import select, update
+from sqlalchemy.dialects.sqlite import dialect as get_sqlite_dialect
+from sqlalchemy.schema import CreateTable
 
 from dql.data_storage.abstract import AbstractDataStorage
-from dql.dataset import DatasetRecord, DatasetRow
+from dql.dataset import DATASET_ROW_FIELDS, DatasetRecord, DatasetRow
+from dql.dataset import Status as DatasetStatus
+from dql.dataset import dataset_table
 from dql.error import DQLError
 from dql.node import AnyNode, DirType, Node, NodeWithPath
 from dql.storage import Status as StorageStatus
@@ -23,12 +40,17 @@ from dql.utils import GLOB_CHARS, DQLDir, is_expired
 
 from .query import Query, Schema
 
+if TYPE_CHECKING:
+    from sqlalchemy.sql.elements import ColumnClause, CompilerElement, TextClause
+
 logger = logging.getLogger("dql")
 
 RETRY_START_SEC = 0.01
 RETRY_MAX_TIMES = 10
 RETRY_FACTOR = 2
 
+Column = Union[str, "ColumnClause[Any]", "TextClause"]
+
 # sqlite 3.31.1 is the earliest version tested in CI
 if sqlite3.sqlite_version_info < (3, 31, 1):
     logger.warning(
@@ -37,6 +59,32 @@ if sqlite3.sqlite_version_info < (3, 31, 1):
     )
 
 
+sqlite_dialect = get_sqlite_dialect(paramstyle="named")
+
+
+def compile_statement(
+    statement: "CompilerElement",
+) -> Union[Tuple[str], Tuple[str, Dict[str, Any]]]:
+    compiled = statement.compile(dialect=sqlite_dialect)
+    if compiled.params is None:
+        return (compiled.string,)
+    return compiled.string, compiled.params
+
+
+def select_table(
+    table: str,
+    columns: Optional[Sequence[Column]] = None,
+):
+    column_objects: List[Union["ColumnClause[Any]", "TextClause"]]
+    if columns is None:
+        column_objects = [sqlalchemy.text("*")]
+    else:
+        column_objects = [
+            sqlalchemy.column(c) if isinstance(c, str) else c for c in columns
+        ]
+    return select(*column_objects).select_from(sqlalchemy.table(table))
+
+
 def adapt_datetime(val: datetime) -> str:
     if not (val.tzinfo is timezone.utc or val.tzname() == "UTC"):
         try:
@@ -93,24 +141,9 @@ DATASET_FIELDS = [
     "description",
     "version",
     "labels",
-]
-
-DATASET_ROW_FIELDS = [
-    "id",
-    "dir_type",
-    "parent_id",
-    "name",
-    "checksum",
-    "etag",
-    "version",
-    "is_latest",
-    "last_modified",
-    "size",
-    "owner_name",
-    "owner_id",
-    "path_str",
-    "anno",
-    "source",
+    "status",
+    "created_at",
+    "finished_at",
 ]
 
 PATH_STR_INDEX = NodeSchema.fields.index("path_str")
@@ -195,7 +228,7 @@ class SQLiteDataStorage(AbstractDataStorage):
             else:
                 self.db = sqlite3.connect(self.db_file, detect_types=detect_types)
             self.engine = sqlalchemy.create_engine(
-                "sqlite+pysqlite:///", creator=lambda: self.db
+                "sqlite+pysqlite:///", creator=lambda: self.db, future=True
             )
 
             self.db.isolation_level = None  # Use autocommit mode
@@ -205,6 +238,8 @@ class SQLiteDataStorage(AbstractDataStorage):
             self.db.execute("PRAGMA journal_mode = WAL")
             self.db.execute("PRAGMA synchronous = NORMAL")
             self.db.execute("PRAGMA case_sensitive_like = ON")
+            if os.environ.get("DEBUG_SHOW_SQL_QUERIES"):
+                self.db.set_trace_callback(print)
 
             self._init_storage_table()
             self._init_datasets_tables()
@@ -249,7 +284,10 @@ class SQLiteDataStorage(AbstractDataStorage):
                 name            TEXT NOT NULL UNIQUE,
                 description     TEXT,
                 labels          JSON DEFAULT('[]'),
-                shadow          BOOL NOT NULL
+                shadow          BOOL NOT NULL,
+                status          INTEGER NOT NULL,
+                created_at      DATETIME,
+                finished_at    DATETIME
             );
             CREATE TABLE IF NOT EXISTS datasets_versions
             (
@@ -479,6 +517,9 @@ class SQLiteDataStorage(AbstractDataStorage):
                 datasets.description,
                 datasets.labels,
                 datasets.shadow,
+                datasets.status,
+                datasets.created_at,
+                datasets.finished_at,
                 datasets_versions.version
             FROM datasets
             LEFT JOIN datasets_versions ON datasets.id = datasets_versions.dataset_id
@@ -491,69 +532,71 @@ class SQLiteDataStorage(AbstractDataStorage):
             if dataset:
                 yield dataset
 
-    def create_dataset_rows_table(self, name):
-        # creates special shadow dataset table to store rows
-        self.db.execute(
-            f"""
-            CREATE TABLE IF NOT EXISTS {name}
-            (
-                id              INTEGER PRIMARY KEY,
-                dir_type        INTEGER,
-                parent_id       INTEGER,
-                name            TEXT NOT NULL,
-                checksum        TEXT,
-                etag            TEXT,
-                version         TEXT,
-                is_latest       BOOL,
-                last_modified   DATETIME,
-                size            BIGINT NOT NULL,
-                owner_name      TEXT,
-                owner_id        TEXT,
-                path_str        TEXT,
-                anno            JSON,
-                source          TEXT NOT NULL,
-                UNIQUE(source, path_str)
-            );
-            """
+    def create_dataset_rows_table(
+        self,
+        name: str,
+        custom_columns: Sequence["sqlalchemy.Column"] = (),
+        if_not_exists: bool = True,
+    ):
+        q = CreateTable(
+            dataset_table(name, custom_columns=custom_columns),
+            if_not_exists=if_not_exists,
         )
+        self.db.execute(*compile_statement(q))
 
-    def get_dataset_rows(
-        self, name: str, limit=20, version=None
-    ) -> Iterable[DatasetRow]:
+    def get_dataset_row_values(
+        self,
+        name: str,
+        columns: Optional[Sequence[Column]] = None,
+        limit=20,
+        version=None,
+    ) -> Iterable[Mapping[str, Any]]:
         dataset = self.get_dataset(name)
         assert dataset
         dataset_table_name = self._dataset_table_name(dataset.id, version)
 
-        limit_q = ""
-        args = []
+        q = select_table(dataset_table_name, columns=columns)
         if limit:
-            limit_q = "LIMIT ?"
-            args.append(limit)
+            q = q.limit(limit)
 
-        for row in self.db.execute(
-            f"""SELECT * FROM {dataset_table_name} {limit_q}""",
-            args,
-        ).fetchall():
-            yield DatasetRow(*row)
+        cur = self.db.cursor()
+        cur.row_factory = sqlite3.Row  # type: ignore[assignment]
+        yield from cur.execute(*compile_statement(q)).fetchall()
 
-    def create_shadow_dataset(self, name: str, create_rows: bool = True) -> None:
+    def get_dataset_rows(
+        self, name: str, limit=20, version=None
+    ) -> Iterable[DatasetRow]:
+        for row in self.get_dataset_row_values(
+            name, columns=DATASET_ROW_FIELDS, limit=limit, version=version
+        ):
+            yield DatasetRow(**row)
+
+    def create_shadow_dataset(
+        self, name: str, create_rows: Optional[bool] = True
+    ) -> DatasetRecord:
         """Creates new shadow dataset if it doesn't exist yet"""
         with self.db:
             self.db.execute("begin")
 
             # adds entry into datasets table
             self.db.execute(
-                """INSERT INTO datasets(name, shadow) VALUES (?, ?)
+                """
+                INSERT INTO datasets(name, shadow, status, created_at)
+                VALUES (?, ?, ?, ?)
                 ON CONFLICT(name) DO NOTHING
                 """,
-                [name, True],
+                [name, True, DatasetStatus.CREATED, datetime.now(timezone.utc)],
             )
             dataset = self.get_dataset(name)
             assert dataset
             assert dataset.shadow
+
             if create_rows:
                 table_name = self._dataset_table_name(dataset.id)
                 self.create_dataset_rows_table(table_name)
+                dataset = self.update_dataset_status(dataset, DatasetStatus.PENDING)
+
+            return dataset  # type: ignore[return-value]
 
     def insert_into_shadow_dataset(
         self, name: str, uri: str, path: str, recursive=False
@@ -568,8 +611,16 @@ class SQLiteDataStorage(AbstractDataStorage):
         self.uri = uri  # needed for parent node and .nodes
         if recursive:
             if not path.endswith("*"):
-                path = path.rstrip("/") + "/*"  # glob filter must end with /*
-            nodes = self.nodes.where(valid=True, path_str__glob=path, type="file")
+                path = path.rstrip("/")
+                if path:
+                    # setting sufix for globs only if path is defined
+                    path = path + "/*"  # glob filter must end with /*
+
+            if path:
+                nodes = self.nodes.where(valid=True, path_str__glob=path, type="file")
+            else:
+                # we are getting the whole storage, no need to query by path
+                nodes = self.nodes.where(valid=True, type="file")
         else:
             parent = self.get_node_by_path(path.lstrip("/").rstrip("/*"))
             nodes = self.nodes.where(valid=True, parent_id=parent.id, type="file")
@@ -618,7 +669,6 @@ class SQLiteDataStorage(AbstractDataStorage):
             insert_q,
             [uri] + where_params,
         )
-        self.db.commit()
 
     def rename_dataset_table(
         self,
@@ -721,6 +771,9 @@ class SQLiteDataStorage(AbstractDataStorage):
                 datasets.description,
                 datasets.labels,
                 datasets.shadow,
+                datasets.status,
+                datasets.created_at,
+                datasets.finished_at,
                 datasets_versions.version
             FROM datasets
             LEFT JOIN datasets_versions ON datasets.id = datasets_versions.dataset_id
@@ -948,6 +1001,19 @@ class SQLiteDataStorage(AbstractDataStorage):
                         timestamp=end_time, expires=expires
                     )
 
+    @retry_sqlite_locks
+    def update_dataset_status(
+        self, dataset: DatasetRecord, status: int
+    ) -> DatasetRecord:
+        update_data: Dict[str, Any] = {"status": status}
+        if status in [DatasetStatus.COMPLETE, DatasetStatus.FAILED]:
+            # if in final state, updating finished_at datetime
+            update_data["finished_at"] = datetime.now(timezone.utc)
+
+        self.update_dataset(dataset.name, **update_data)
+        dataset.update(**update_data)
+        return dataset
+
     def validate_paths(self) -> int:
         assert isinstance(self.table_name, str), "table_name must be a string"
         t1 = self.table.alias("t1")
@@ -968,8 +1034,7 @@ class SQLiteDataStorage(AbstractDataStorage):
             .where(t1.c.id.in_(id_query))  # type: ignore[attr-defined]
         )
 
-        with self.engine.connect() as con:
-            row_count = con.execute(query).rowcount
+        row_count = self.db.execute(*compile_statement(query)).rowcount
         if row_count:
             logger.warning(
                 "File names that collide with directory names will be ignored. "
