@@ -1,12 +1,33 @@
 import json
 from dataclasses import dataclass
-from typing import List, NamedTuple, Optional, Sequence, Type, TypeVar
+from datetime import datetime
+from typing import List, Optional, Sequence, Type, TypeVar
 
-from dql.node import _fields
+import attrs
+from sqlalchemy import (
+    JSON,
+    BigInteger,
+    Boolean,
+    Column,
+    DateTime,
+    Integer,
+    MetaData,
+    Table,
+    Text,
+    UniqueConstraint,
+)
 
 T = TypeVar("T", bound="DatasetRecord")
 
 
+class Status:
+    CREATED = 1
+    PENDING = 2
+    FAILED = 3
+    COMPLETE = 4
+    STALE = 6
+
+
 @dataclass
 class DatasetRecord:
     id: int
@@ -15,6 +36,9 @@ class DatasetRecord:
     labels: Sequence[str]
     shadow: bool
     versions: Optional[List[int]]
+    status: int = Status.CREATED
+    created_at: Optional[datetime] = None
+    finished_at: Optional[datetime] = None
 
     @classmethod
     def parse(
@@ -24,6 +48,9 @@ class DatasetRecord:
         description: Optional[str],
         labels: str,
         shadow: int,
+        status: int,
+        created_at: datetime,
+        finished_at: Optional[datetime],
         version: Optional[int],
     ) -> T:
         labels_lst: List[str] = json.loads(labels)
@@ -31,7 +58,22 @@ class DatasetRecord:
         if version:
             versions = [version]
 
-        return cls(id, name, description, labels_lst, bool(shadow), versions)
+        return cls(
+            id,
+            name,
+            description,
+            labels_lst,
+            bool(shadow),
+            versions,
+            status,
+            created_at,
+            finished_at,
+        )
+
+    def update(self, **kwargs):
+        for key, value in kwargs.items():
+            if hasattr(self, key):
+                setattr(self, key, value)
 
     def merge_versions(self, other: "DatasetRecord") -> None:
         """Merge versions from another dataset"""
@@ -92,10 +134,53 @@ class DatasetRecord:
         return sorted(self.versions)[-2]
 
 
-fields = [f for f in _fields if f[0] != "valid"]
-# TODO use DatasetRowSchema when dataset query refactoring is done (merged)
-# https://github.com/iterative/dql/pull/258
-DatasetRow = NamedTuple(  # type: ignore
-    "DatasetRow",
-    fields + [("source", str)],
-)
+@attrs.define
+class DatasetRow:
+    id: int
+    dir_type: int
+    parent_id: Optional[int]
+    name: str
+    checksum: str
+    etag: str
+    version: str
+    is_latest: bool
+    last_modified: Optional[datetime]
+    size: int
+    owner_name: str
+    owner_id: str
+    path_str: str
+    anno: Optional[str]
+    source: str
+
+
+def dataset_table(name: str, custom_columns: Sequence["Column"] = ()):
+    return Table(
+        name,
+        MetaData(),
+        *core_dataset_columns(),
+        UniqueConstraint("source", "path_str"),
+        *custom_columns,
+    )
+
+
+def core_dataset_columns() -> List["Column"]:
+    return [
+        Column("id", Integer, primary_key=True),
+        Column("dir_type", Integer),
+        Column("parent_id", Integer),
+        Column("name", Text, nullable=False),
+        Column("checksum", Text),
+        Column("etag", Text),
+        Column("version", Text),
+        Column("is_latest", Boolean),
+        Column("last_modified", DateTime),
+        Column("size", BigInteger, nullable=False),
+        Column("owner_name", Text),
+        Column("owner_id", Text),
+        Column("path_str", Text),
+        Column("anno", JSON),
+        Column("source", Text, nullable=False),
+    ]
+
+
+DATASET_ROW_FIELDS = tuple([f.name for f in attrs.fields(DatasetRow)])
